@@ -20,6 +20,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "mainwindow.h"
 #include "../common/ostprotolib.h"
 #include "../common/protocolmanager.h"
+#include "../common/protocolwidgetfactory.h"
+#include "preferences.h"
 #include "settings.h"
 
 #include <QApplication>
@@ -31,33 +33,23 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 extern const char* version;
 extern const char* revision;
 extern ProtocolManager *OstProtocolManager;
+extern ProtocolWidgetFactory *OstProtocolWidgetFactory;
 
 QSettings *appSettings;
 QMainWindow *mainWindow;
 
-#if defined(Q_OS_WIN32)
-QString kGzipPathDefaultValue;
-QString kDiffPathDefaultValue;
-QString kAwkPathDefaultValue;
-#endif
-
 int main(int argc, char* argv[])
 {
     QApplication app(argc, argv);
     int exitCode;
 
-#if defined(Q_OS_WIN32)
-    kGzipPathDefaultValue = app.applicationDirPath() + "/gzip.exe";
-    kDiffPathDefaultValue = app.applicationDirPath() + "/diff.exe";
-    kAwkPathDefaultValue = app.applicationDirPath() + "/gawk.exe";
-#endif
-
     app.setApplicationName("Ostinato");
     app.setOrganizationName("Ostinato");
     app.setProperty("version", version);
     app.setProperty("revision", revision);
 
     OstProtocolManager = new ProtocolManager();
+    OstProtocolWidgetFactory = new ProtocolWidgetFactory();
 
     /* (Portable Mode) If we have a .ini file in the same directory as the 
        executable, we use that instead of the platform specific location
@@ -75,6 +67,8 @@ int main(int argc, char* argv[])
         appSettings->value(kDiffPathKey, kDiffPathDefaultValue).toString(),
         appSettings->value(kAwkPathKey, kAwkPathDefaultValue).toString());
 
+    Preferences::initDefaults();
+
     mainWindow = new MainWindow;
     mainWindow->show();
     exitCode =  app.exec();
