@@ -33,6 +33,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 using ::google::protobuf::NewCallback;
 
 extern QMainWindow *mainWindow;
+extern char *version;
 
 quint32 PortGroup::mPortGroupAllocId = 0;
 
@@ -47,6 +48,8 @@ PortGroup::PortGroup(QHostAddress ip, quint16 port)
     statsController = new PbRpcController(portIdList_, portStatsList_);
     isGetStatsPending_ = false;
 
+    compat = kUnknown;
+
     reconnect = false;
     reconnectAfter = kMinReconnectWaitTime;
     reconnectTimer = new QTimer(this);
@@ -54,7 +57,8 @@ PortGroup::PortGroup(QHostAddress ip, quint16 port)
     connect(reconnectTimer, SIGNAL(timeout()), 
         this, SLOT(on_reconnectTimer_timeout()));
 
-    rpcChannel = new PbRpcChannel(ip, port);
+    rpcChannel = new PbRpcChannel(ip, port, 
+                                  OstProto::Notification::default_instance());
     serviceStub = new OstProto::OstService::Stub(rpcChannel);
 
     // FIXME(LOW):Can't for my life figure out why this ain't working!
@@ -68,6 +72,11 @@ PortGroup::PortGroup(QHostAddress ip, quint16 port)
     connect(rpcChannel, SIGNAL(error(QAbstractSocket::SocketError)), 
         this, SLOT(on_rpcChannel_error(QAbstractSocket::SocketError)));
 
+    connect(rpcChannel, 
+        SIGNAL(notification(int, ::google::protobuf::Message*)), 
+        this, 
+        SLOT(on_rpcChannel_notification(int, ::google::protobuf::Message*)));
+
     connect(this, SIGNAL(portListChanged(quint32)),
         this, SLOT(when_portListChanged(quint32)), Qt::QueuedConnection);
 }
@@ -112,19 +121,63 @@ void PortGroup::on_rpcChannel_stateChanged(QAbstractSocket::SocketState state)
 
 void PortGroup::on_rpcChannel_connected()
 {
-    OstProto::Void *void_ = new OstProto::Void;
-    OstProto::PortIdList *portIdList = new OstProto::PortIdList;
+    OstProto::VersionInfo *verInfo = new OstProto::VersionInfo;
+    OstProto::VersionCompatibility *verCompat = 
+            new OstProto::VersionCompatibility;
     
     qDebug("connected\n");
     emit portGroupDataChanged(mPortGroupId);
 
     reconnectAfter = kMinReconnectWaitTime;
 
-    qDebug("requesting portlist ...");
+    qDebug("requesting version check ...");
+    verInfo->set_client_name("ostinato");
+    verInfo->set_version(version);
     
-    PbRpcController *controller = new PbRpcController(void_, portIdList);
-    serviceStub->getPortIdList(controller, void_, portIdList, 
-            NewCallback(this, &PortGroup::processPortIdList, controller));
+    PbRpcController *controller = new PbRpcController(verInfo, verCompat);
+    serviceStub->checkVersion(controller, verInfo, verCompat, 
+            NewCallback(this, &PortGroup::processVersionCompatibility, 
+                        controller));
+}
+
+void PortGroup::processVersionCompatibility(PbRpcController *controller)
+{
+    OstProto::VersionCompatibility *verCompat 
+        = static_cast<OstProto::VersionCompatibility*>(controller->response());
+
+    Q_ASSERT(verCompat != NULL);
+
+    qDebug("got version result ...");
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
+        goto _error_exit;
+    }
+
+    if (verCompat->result() == OstProto::VersionCompatibility::kIncompatible) {
+        qWarning("incompatible version %s (%s)", version, 
+                qPrintable(QString::fromStdString(verCompat->notes())));
+        compat = kIncompatible;
+        emit portGroupDataChanged(mPortGroupId);
+        goto _error_exit;
+    }
+
+    compat = kCompatible;
+
+    {
+        OstProto::Void *void_ = new OstProto::Void;
+        OstProto::PortIdList *portIdList = new OstProto::PortIdList;
+
+        qDebug("requesting portlist ...");
+        PbRpcController *controller = new PbRpcController(void_, portIdList);
+        serviceStub->getPortIdList(controller, void_, portIdList, 
+                NewCallback(this, &PortGroup::processPortIdList, controller));
+    }
+
+_error_exit:
+    delete controller;
 }
 
 void PortGroup::on_rpcChannel_disconnected()
@@ -138,6 +191,8 @@ void PortGroup::on_rpcChannel_disconnected()
     emit portListChanged(mPortGroupId);
     emit portGroupDataChanged(mPortGroupId);
 
+    isGetStatsPending_ = false;
+
     if (reconnect)
     {
         qDebug("starting reconnect timer for %d ms ...", reconnectAfter);
@@ -150,6 +205,9 @@ void PortGroup::on_rpcChannel_error(QAbstractSocket::SocketError socketError)
     qDebug("%s: error %d", __FUNCTION__, socketError);
     emit portGroupDataChanged(mPortGroupId);
 
+    if (socketError == QAbstractSocket::RemoteHostClosedError)
+        reconnect = false;
+
     qDebug("%s: state %d", __FUNCTION__, rpcChannel->state());
     if ((rpcChannel->state() == QAbstractSocket::UnconnectedState) && reconnect)
     {
@@ -158,6 +216,50 @@ void PortGroup::on_rpcChannel_error(QAbstractSocket::SocketError socketError)
     }
 }
 
+void PortGroup::on_rpcChannel_notification(int notifType, 
+        ::google::protobuf::Message *notification)
+{
+    OstProto::Notification *notif = 
+        dynamic_cast<OstProto::Notification*>(notification);
+
+    if (!notif) {
+        qWarning("unable to dynamic cast notif");
+        return;
+    }
+
+    if (notifType != notif->notif_type()) {
+        qWarning("notif type mismatch %d/%d msg = %s",
+                notifType, notif->notif_type(),
+                notification->DebugString().c_str());
+        return;
+    }
+
+    switch (notifType) 
+    {
+        case OstProto::portConfigChanged: {
+
+            if (!notif->port_id_list().port_id_size()) {
+                qWarning("notif(portConfigChanged) has an empty port_id_list");
+                return;
+            }
+
+            OstProto::PortIdList *portIdList = new OstProto::PortIdList;
+            OstProto::PortConfigList *portConfigList = 
+                                            new OstProto::PortConfigList;
+            PbRpcController *controller = new PbRpcController(portIdList, 
+                                                           portConfigList);
+
+            portIdList->CopyFrom(notif->port_id_list());
+            serviceStub->getPortConfig(controller, portIdList, portConfigList, 
+                NewCallback(this, &PortGroup::processUpdatedPortConfig, 
+                    controller));
+            break;
+        }
+        default:
+            break;
+    }
+}
+
 void PortGroup::when_portListChanged(quint32 /*portGroupId*/)
 {
     if (state() == QAbstractSocket::ConnectedState && numPorts() <= 0)
@@ -186,7 +288,8 @@ void PortGroup::processPortIdList(PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
         goto _error_exit;
     }
 
@@ -237,7 +340,8 @@ void PortGroup::processPortConfigList(PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
         goto _error_exit;
     }
 
@@ -252,10 +356,14 @@ void PortGroup::processPortConfigList(PbRpcController *controller)
         mPorts[id]->updatePortConfig(portConfigList->mutable_port(i));
     }
 
-    //emit portListChanged(mPortGroupId);
-
-    // FIXME: check if we need new signals since we are not changing the
-    // number of ports, just the port data
+    // FIXME: Ideally we should emit portGroupDataChanged since only 
+    // port data is changing; but writing the corresponding slot in 
+    // PortStatsModel for that signal turned out to be very bug prone 
+    // causing assert failures when portgroups are added/deleted or
+    // connected/disconnected in different orders
+    // TODO: Revisit this when we refactor the domain-objects/model/view 
+    // design
+    emit portListChanged(mPortGroupId);
 
     if (numPorts() > 0)
         getStreamIdList();
@@ -367,25 +475,12 @@ void PortGroup::processModifyPortAck(PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
-        goto _exit;
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
     }
 
-    {
-        OstProto::PortIdList *portIdList = new OstProto::PortIdList;
-        OstProto::PortConfigList *portConfigList = new OstProto::PortConfigList;
-        PbRpcController *controller2 = new PbRpcController(portIdList, 
-                portConfigList);
-
-        OstProto::PortId *portId = portIdList->add_port_id();
-        portId->CopyFrom(static_cast<OstProto::PortConfigList*>
-                (controller->request())->mutable_port(0)->port_id());
-
-        serviceStub->getPortConfig(controller, portIdList, portConfigList, 
-            NewCallback(this, &PortGroup::processUpdatedPortConfig, 
-                controller2));
-    }
-_exit:
+    mainWindow->setEnabled(true);
+    QApplication::restoreOverrideCursor();
     delete controller;
 }
 
@@ -398,13 +493,11 @@ void PortGroup::processUpdatedPortConfig(PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
         goto _exit;
     }
 
-    if (portConfigList->port_size() != 1)
-        qDebug("port size = %d (expected = 1)", portConfigList->port_size());
-
     for(int i = 0; i < portConfigList->port_size(); i++)
     {
         uint    id;
@@ -418,8 +511,6 @@ void PortGroup::processUpdatedPortConfig(PbRpcController *controller)
 
 
 _exit:
-    mainWindow->setEnabled(true);
-    QApplication::restoreOverrideCursor();
     delete controller;
 }
 
@@ -448,7 +539,8 @@ void PortGroup::processStreamIdList(int portIndex, PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
         goto _exit;
     }
 
@@ -519,7 +611,8 @@ void PortGroup::processStreamConfigList(int portIndex,
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(controller->ErrorString()));
         goto _exit;
     }
 
@@ -572,7 +665,7 @@ void PortGroup::startTx(QList<uint> *portList)
             portId->set_id(portList->at(i));
         }
 
-        serviceStub->startTx(controller, portIdList, ack,
+        serviceStub->startTransmit(controller, portIdList, ack,
                 NewCallback(this, &PortGroup::processStartTxAck, controller));
     }
 _exit:
@@ -607,7 +700,7 @@ void PortGroup::stopTx(QList<uint> *portList)
             portId->set_id(portList->at(i));
         }
 
-        serviceStub->stopTx(controller, portIdList, ack,
+        serviceStub->stopTransmit(controller, portIdList, ack,
                 NewCallback(this, &PortGroup::processStopTxAck, controller));
     }
 _exit:
@@ -779,7 +872,8 @@ void PortGroup::processPortStatsList()
 
     if (statsController->Failed())
     {
-        qDebug("%s: rpc failed", __FUNCTION__);
+        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+                qPrintable(statsController->ErrorString()));
         goto _error_exit;
     }
 
