@@ -26,6 +26,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "portgrouplist.h"
 #include "portstatsmodel.h"
 
+class QSortFilterProxyModel;
+
 class PortStatsWindow : public QWidget, public Ui::PortStatsWindow
 {
     Q_OBJECT
@@ -34,9 +36,8 @@ public:
     PortStatsWindow(PortGroupList *pgl, QWidget *parent = 0);
     ~PortStatsWindow();
 
-private:
-    PortGroupList        *pgl;
-    PortStatsModel        *model;
+public slots:
+    void showMyReservedPortsOnly(bool enabled);
 
 private slots:
     void on_tbStartTransmit_clicked();
@@ -50,6 +51,14 @@ private slots:
     void on_tbClearAll_clicked();
 
     void on_tbFilter_clicked();
+
+private:
+    QModelIndexList selectedColumns();
+
+    PortGroupList           *pgl;
+    PortStatsModel          *model;
+    QSortFilterProxyModel   *proxyStatsModel;
+
 };
 
 #endif
