@@ -22,13 +22,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "streamconfigdialog.h"
 #include "stream.h"
 #include "abstractprotocol.h"
+#include "abstractprotocolconfig.h"
 #include "protocollistiterator.h"
 
 #include "modeltest.h"
 
-// FIXME(HI) - remove
 #include "../common/protocolmanager.h"
+#include "../common/protocolwidgetfactory.h"
+
 extern ProtocolManager *OstProtocolManager;
+extern ProtocolWidgetFactory *OstProtocolWidgetFactory;
 
 QRect StreamConfigDialog::lastGeometry;
 int StreamConfigDialog::lastTopLevelTabIndex = 0;
@@ -133,8 +136,10 @@ StreamConfigDialog::StreamConfigDialog(Port &port, uint streamIndex,
     mpAvailableProtocolsModel = new QStringListModel(
         OstProtocolManager->protocolDatabase(), this);
     lvAllProtocols->setModel(mpAvailableProtocolsModel);
+    lvAllProtocols->setEditTriggers(QAbstractItemView::NoEditTriggers);
     mpSelectedProtocolsModel = new QStringListModel(this);
     lvSelectedProtocols->setModel(mpSelectedProtocolsModel);
+    lvSelectedProtocols->setEditTriggers(QAbstractItemView::NoEditTriggers);
 
 
     connect(lvAllProtocols->selectionModel(),
@@ -146,6 +151,8 @@ StreamConfigDialog::StreamConfigDialog(Port &port, uint streamIndex,
         this, SLOT(when_lvSelectedProtocols_currentChanged(const QModelIndex&,
             const QModelIndex&)));
 
+    variableFieldsWidget->setStream(mpStream);
+
     LoadCurrentStream();
     mpPacketModel = new PacketModel(this);
     tvPacketTree->setModel(mpPacketModel);
@@ -306,10 +313,60 @@ StreamConfigDialog::~StreamConfigDialog()
     for (int i = ProtoMin; i < ProtoMax; i++)
         delete bgProto[i];
 
+    foreach (AbstractProtocolConfigForm* w, _protocolWidgets) {
+        OstProtocolWidgetFactory->deleteConfigWidget(w);
+    }
+
     delete _iter;
     delete mpStream;
 }
 
+void StreamConfigDialog::loadProtocolWidgets()
+{
+    ProtocolListIterator    *iter;
+
+    // NOTE: Protocol Widgets are created on demand. Once created we
+    //       store them in _protocolWidgets indexed by the protocol
+    //       object's address (to ensure unique widgets for multiple
+    //       objects of the same class). Subsequently we check
+    //       _protocolWidgets before creating a new widget
+    iter = mpStream->createProtocolListIterator();
+    while (iter->hasNext())
+    {
+        AbstractProtocol* p = iter->next();
+        AbstractProtocolConfigForm *w = _protocolWidgets.value(p);
+        if (!w) {
+            w = OstProtocolWidgetFactory->createConfigWidget(
+                    p->protocolNumber());
+            _protocolWidgets.insert(p, w);
+        }
+        w->loadWidget(p);
+    }
+    delete iter;
+}
+
+void StreamConfigDialog::storeProtocolWidgets()
+{
+    ProtocolListIterator    *iter;
+
+    // NOTE: After creating a widget, we need to call loadWidget()
+    //       to load the protocol's default values
+    iter = mpStream->createProtocolListIterator();
+    while (iter->hasNext())
+    {
+        AbstractProtocol* p = iter->next();
+        AbstractProtocolConfigForm *w = _protocolWidgets.value(p);
+        if (!w) {
+            w = OstProtocolWidgetFactory->createConfigWidget(
+                    p->protocolNumber());
+            w->loadWidget(p);
+            _protocolWidgets.insert(p, w);
+        }
+        w->storeWidget(p);
+    }
+    delete iter;
+}
+
 void StreamConfigDialog::on_cmbPktLenMode_currentIndexChanged(QString mode)
 {
     if (mode == "Fixed")
@@ -461,6 +518,8 @@ void StreamConfigDialog::on_tbDelete_clicked()
 
     Q_CHECK_PTR(p);
     _iter->remove();
+    // Free both protocol and associated widget
+    delete _protocolWidgets.take(p);
     delete p;
 
     updateSelectProtocolsAdvancedWidget();
@@ -565,12 +624,19 @@ void StreamConfigDialog::on_twTopLevel_currentChanged(int index)
                 w->setParent(0);
             }
 
-            // Repopulate the widgets
+            // Repopulate the widgets - create new ones, if required
             _iter->toFront();
             while (_iter->hasNext())
             {
                 AbstractProtocol* p = _iter->next();
-                tbProtocolData->addItem(p->configWidget(), p->name());
+                AbstractProtocolConfigForm *w = _protocolWidgets.value(p);
+                if (!w) {
+                    w = OstProtocolWidgetFactory->createConfigWidget(
+                            p->protocolNumber());
+                    w->loadWidget(p);
+                    _protocolWidgets.insert(p, w);
+                }
+                tbProtocolData->addItem(w, p->name());
             }
 
             if (lastProtocolDataIndex < tbProtocolData->count())
@@ -580,15 +646,26 @@ void StreamConfigDialog::on_twTopLevel_currentChanged(int index)
             break;
         }
 
-        // Stream Control
+        // Variable Fields
         case 2:
+        {
+            StoreCurrentStream();
+
+            // Stream protocols may have changed - clear and reload
+            variableFieldsWidget->clear();
+            variableFieldsWidget->load();
+            break;
+        }
+
+        // Stream Control
+        case 3:
         {
             StoreCurrentStream();
             break;
         }
 
         // Packet View
-        case 3:
+        case 4:
         {
             StoreCurrentStream();
             mpPacketModel->setSelectedProtocols(*_iter);
@@ -751,6 +828,7 @@ void StreamConfigDialog::__updateProtocol(int level, int newId)
 
         ret = skipProtocols(level-1);
         Q_ASSERT(ret == true);
+        Q_UNUSED(ret);
 
         Q_ASSERT(oldId != newId);
         Q_ASSERT(newId != ButtonIdOther);
@@ -772,6 +850,8 @@ void StreamConfigDialog::__updateProtocol(int level, int newId)
                             newId, mpStream));
                 else
                     _iter->remove();
+                // Free both protocol and associated widget
+                delete _protocolWidgets.take(p);
                 delete p;
                 if (level == ProtoPayload)
                 {
@@ -779,6 +859,8 @@ void StreamConfigDialog::__updateProtocol(int level, int newId)
                     {
                         p = _iter->next();
                         _iter->remove();
+                        // Free both protocol and associated widget
+                        delete _protocolWidgets.take(p);
                         delete p;
                     }
                 }
@@ -892,7 +974,12 @@ void StreamConfigDialog::LoadCurrentStream()
         updateSelectProtocolsSimpleWidget();
         updateSelectProtocolsAdvancedWidget();
 
-        mpStream->loadProtocolWidgets();
+        loadProtocolWidgets();
+    }
+
+    // Variable Fields
+    {
+        variableFieldsWidget->load();
     }
 
     // Stream Control
@@ -967,7 +1054,12 @@ void StreamConfigDialog::StoreCurrentStream()
 
     // Protocols
     {
-        pStream->storeProtocolWidgets();
+        storeProtocolWidgets();
+    }
+
+    // Variable Fields
+    {
+        variableFieldsWidget->store();
     }
 
     // Stream Control
@@ -1005,8 +1097,8 @@ void StreamConfigDialog::on_tbProtocolData_currentChanged(int /*index*/)
     // protocols e.g. TCP/UDP port numbers are dependent on Port/Protocol 
     // selection in TextProtocol
 #if 0 // FIXME: temp mask to avoid crash till we fix it
-    mpStream->storeProtocolWidgets();
-    mpStream->loadProtocolWidgets();
+    storeProtocolWidgets();
+    loadProtocolWidgets();
 #endif
 }
 
