@@ -17,11 +17,12 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 
-#include <qendian.h>
-
 #include "abstractprotocol.h" 
-#include "streambase.h"
+
 #include "protocollistiterator.h"
+#include "streambase.h"
+
+#include <qendian.h>
 
 /*!
   \class AbstractProtocol
@@ -46,16 +47,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
   - fieldFlags()
   - fieldData()
   - setFieldData()
-  - configWidget() [pure virtual]
-  - loadConfigWidget() [pure virtual]
-  - storeConfigWidget() [pure virtual]
 
   Depending on certain conditions, subclasses may need to reimplement the
   following additional methods -
   - protocolIdType()
   - protocolId()
   - protocolFrameSize()
-  - isProtocolFrameValueVariable()
   - isProtocolFrameSizeVariable()
   - protocolFrameVariableCount()
 
@@ -79,8 +76,10 @@ AbstractProtocol::AbstractProtocol(StreamBase *stream, AbstractProtocol *parent)
     prev = next = NULL;
     _metaFieldCount = -1;
     _frameFieldCount = -1;
+    _frameVariableCount = -1;
     protoSize = -1;
     _hasPayload = true;
+    _cacheFlags |= FieldFrameBitOffsetCache;
 }
 
 /*!
@@ -107,6 +106,8 @@ AbstractProtocol* AbstractProtocol::createInstance(StreamBase* /* stream */,
   (file: protocol.proto)
 
   Subclasses MUST implement this function
+
+  \todo convert this to a protected data member instead of a virtual function
 */
 quint32 AbstractProtocol::protocolNumber() const
 {
@@ -114,6 +115,42 @@ quint32 AbstractProtocol::protocolNumber() const
     return 0xFFFFFFFF;
 }
 
+/*!
+ Copies the common data (not specific to individual protocols) in the 
+ protocol member protobuf data into the passed in protocol parameter.
+ The individual protocol specific protobuf data is copied using 
+ protoDataCopyInto()
+*/
+void AbstractProtocol::commonProtoDataCopyInto(OstProto::Protocol &protocol) const
+{
+    protocol.clear_variable_field();
+    for (int i = 0; i < _data.variable_field_size(); i++)
+    {
+        OstProto::VariableField *vf;
+
+        vf = protocol.add_variable_field();
+        vf->CopyFrom(_data.variable_field(i));
+    }
+}
+
+/*!
+ Copies the common data (not specific to individual protocols) from the 
+ passed in param protocol protobuf into the member protobuf data.
+ The individual protocol specific protobuf data is copied using 
+ protoDataCopyFrom()
+*/
+void AbstractProtocol::commonProtoDataCopyFrom(const OstProto::Protocol &protocol)
+{
+    _data.clear_variable_field();
+    for (int i = 0; i < protocol.variable_field_size(); i++)
+    {
+        OstProto::VariableField *vf;
+
+        vf = _data.add_variable_field();
+        vf->CopyFrom(protocol.variable_field(i));
+    }
+}
+
 /*!
   \fn virtual void AbstractProtocol::protoDataCopyInto(OstProto::Protocol &protocol) const = 0
 
@@ -316,6 +353,115 @@ bool AbstractProtocol::setFieldData(int /*index*/, const QVariant& /*value*/,
     return false;
 }
 
+/*!
+ * Returns the bit offset where the specified field starts within the 
+ * protocolFrameValue()
+ */
+int AbstractProtocol::fieldFrameBitOffset(int index, int streamIndex) const
+{
+    int ofs = 0;
+
+    if ((index < 0) || (index >= fieldCount()) 
+            || !fieldFlags(index).testFlag(FrameField))
+        return -1;
+
+    // Lookup Cache; if not available calculate and cache (if enabled)
+
+    if (_fieldFrameBitOffset.contains(index)) {
+        ofs = _fieldFrameBitOffset.value(index);
+        goto _exit;
+    }
+
+    for (int i = 0; i < index; i++) {
+        if ((_cacheFlags & FieldFrameBitOffsetCache)
+                && !_fieldFrameBitOffset.contains(i))
+            _fieldFrameBitOffset.insert(i, ofs);
+        ofs += fieldData(i, FieldBitSize, streamIndex).toInt();
+    }
+    if ((_cacheFlags & FieldFrameBitOffsetCache))
+        _fieldFrameBitOffset.insert(index, ofs);
+
+    qDebug("======> ffbo index: %d, ofs: %d", index, ofs);
+_exit:
+    return ofs;
+}
+
+/*!
+ * Returns the count of variableFields in the protocol
+ */
+int AbstractProtocol::variableFieldCount() const
+{
+    return _data.variable_field_size();
+}
+
+/*!
+ * Appends a variableField to the protocol
+ */
+void AbstractProtocol::appendVariableField(const OstProto::VariableField &vf)
+{
+    _data.add_variable_field()->CopyFrom(vf);
+
+    // Update the cached value
+    _frameVariableCount = lcm(_frameVariableCount, vf.count());
+}
+
+/*!
+ * Removes the variableField from the protocol at the specified index
+ */
+void AbstractProtocol::removeVariableField(int index)
+{
+    OstProto::Protocol temp;
+
+    if (index >= _data.variable_field_size()) {
+        qWarning("%s: %s variableField[%d] out of range; count: %d)",
+                __FUNCTION__, qPrintable(shortName()), 
+                index, _data.variable_field_size());
+        return;
+    }
+
+    // TODO: this is inefficient - evaluate using RepeatedPtrField?
+    for (int i = 0; i < _data.variable_field_size(); i++) {
+        if (i == index)
+            continue;
+        temp.add_variable_field()->CopyFrom(_data.variable_field(i));
+    }
+
+    _data.clear_variable_field();
+    _frameVariableCount = 1;
+    for (int i = 0; i < temp.variable_field_size(); i++) {
+        _data.add_variable_field()->CopyFrom(temp.variable_field(i));
+        // Recalculate the cached value
+        _frameVariableCount = lcm(_frameVariableCount,
+                                  _data.variable_field(i).count());
+    }
+}
+
+/*!
+ * Returns the variableField at the specified index as a constant Reference
+ * i.e. read-only
+ */
+const OstProto::VariableField& AbstractProtocol::variableField(int index) const
+{
+    Q_ASSERT(index < _data.variable_field_size());
+
+    return _data.variable_field(index);
+}
+
+/*!
+ * Returns the variableField at the specified index as a mutable pointer.
+ * Changes made via the pointer will be reflected in the protocol
+ */
+OstProto::VariableField* AbstractProtocol::mutableVariableField(int index)
+{
+    if ((index < 0) || (index >= _data.variable_field_size()))
+        return NULL;
+
+    // Invalidate the cached value as the caller may potentially modify it
+    _frameVariableCount = -1;
+
+    return _data.mutable_variable_field(index);
+}
+
 /*!
   Returns the protocolIdType for the protocol
 
@@ -542,16 +688,19 @@ QByteArray AbstractProtocol::protocolFrameValue(int streamIndex, bool forCksum)
         }
     }
 
+    // Overwrite proto with the variable fields, if any
+    for (int i = 0; i < _data.variable_field_size(); i++)
+    {
+        OstProto::VariableField vf = _data.variable_field(i);
+        varyProtocolFrameValue(proto, streamIndex, vf);
+    }
+
     return proto;
 }
 
 /*!
   Returns true if the protocol varies one or more of its fields at run-time,
   false otherwise
-
-  The default implementation returns false. A subclass should reimplement
-  if it has varying fields e.g. an IP protocol that increments/decrements
-  the IP address with every packet  
 */
 bool AbstractProtocol::isProtocolFrameValueVariable() const
 {
@@ -578,14 +727,24 @@ bool AbstractProtocol::isProtocolFrameSizeVariable() const
   fields in the protocol. Use the AbstractProtocol::lcm() static utility
   function to calculate the LCM.
 
-  The default implementation returns 1 implying that the protocol has no
-  varying fields. A subclass should reimplement if it has varying fields 
-  e.g. an IP protocol that increments/decrements the IP address with 
-  every packet  
+  The default implementation returns the LCM of all variableFields
+  A subclass should reimplement if it has varying fields e.g. an IP protocol 
+  that increments/decrements the IP address with every packet.\n
+  Subclasses should call the base class method to retreive the count and
+  do a LCM with the subclass' own varying fields
+
 */
 int AbstractProtocol::protocolFrameVariableCount() const
 {
-    return 1;
+    if (_frameVariableCount > 0)
+        return _frameVariableCount;
+
+    _frameVariableCount = 1;
+    for (int i = 0; i < _data.variable_field_size(); i++)
+        _frameVariableCount = lcm(_frameVariableCount, 
+                                  _data.variable_field(i).count());
+
+    return _frameVariableCount;
 }
 
 /*!
@@ -597,6 +756,10 @@ int AbstractProtocol::protocolFrameVariableCount() const
 */
 bool AbstractProtocol::isProtocolFramePayloadValueVariable() const
 {
+    // TODO: it is simpler to do the following -
+    //     return (protocolFramePayloadVariableCount() > 1)
+    // However, it may be inefficient till the time we cache the 
+    // variable count
     AbstractProtocol *p = next;
 
     while (p)
@@ -838,36 +1001,6 @@ out:
     return (quint16) ~sum;
 }
 
-/*!
-  \fn virtual QWidget* AbstractProtocol::configWidget() = 0;
-
-  Returns the configuration widget for the protocol. The protocol retains
-  ownership of the configuration widget - the caller should not free it.
-
-  In the base class this is a pure virtual function. Subclasses MUST implement 
-  this function. See the SampleProtocol for an example
-*/
-
-/*!
-  \fn virtual void AbstractProtocol::loadConfigWidget() = 0;
-
-  Loads data from the protocol's protobuf into the configuration widget of
-  the protocol
-
-  In the base class this is a pure virtual function. Subclasses MUST implement 
-  this function. See the SampleProtocol for an example
-*/
-
-/*!
-  \fn virtual void AbstractProtocol::storeConfigWidget() = 0;
-
-  Stores data from the configuration widget of the protocol into the protocol's
-  protobuf
-
-  In the base class this is a pure virtual function. Subclasses MUST implement 
-  this function. See the SampleProtocol for an example
-*/
-
 // Stein's binary GCD algo - from wikipedia
 quint64 AbstractProtocol::gcd(quint64 u, quint64 v)
 {
@@ -926,3 +1059,78 @@ quint64 AbstractProtocol::lcm(quint64 u, quint64 v)
     return (u * v)/gcd(u, v);
 }
 
+/* 
+ * XXX: varyCounter() is not a member of AbstractProtocol to avoid
+ * moving it into the header file and thereby keeping the header file
+ * clean
+ */
+template <typename T>
+bool varyCounter(QString protocolName, QByteArray &buf, int frameIndex, 
+                 const OstProto::VariableField &varField)
+{
+    int x = (frameIndex % varField.count()) * varField.step();
+
+    T oldfv, newfv;
+
+    if ((varField.offset() + sizeof(T)) > uint(buf.size()))
+    {
+        qWarning("%s varField ofs %d beyond protocol frame %d - skipping", 
+                qPrintable(protocolName), varField.offset(), buf.size());
+        return false;
+    }
+
+    oldfv = *((T*)((uchar*)buf.constData() + varField.offset()));
+    if (sizeof(T) > sizeof(quint8))
+        oldfv = qFromBigEndian(oldfv);
+
+    switch(varField.mode())
+    {
+        case OstProto::VariableField::kIncrement:
+            newfv = (oldfv & ~varField.mask()) 
+                | ((varField.value() + x) & varField.mask());
+            break;
+        case OstProto::VariableField::kDecrement:
+            newfv = (oldfv & ~varField.mask()) 
+                | ((varField.value() - x) & varField.mask());
+            break;
+        case OstProto::VariableField::kRandom:
+            newfv = (oldfv & ~varField.mask()) 
+                | ((varField.value() + qrand()) & varField.mask());
+            break;
+        default:
+            qWarning("%s Unsupported varField mode %d", 
+                    qPrintable(protocolName), varField.mode());
+            return false;
+    }
+
+    if (sizeof(T) == sizeof(quint8))
+        *((uchar*)buf.constData() + varField.offset()) = newfv;
+    else
+        qToBigEndian(newfv, (uchar*)buf.constData() + varField.offset());
+
+    qDebug("%s varField ofs %d oldfv %x newfv %x", 
+            qPrintable(protocolName), varField.offset(), oldfv, newfv);
+    return true;
+}
+
+void AbstractProtocol::varyProtocolFrameValue(QByteArray &buf, int frameIndex,
+        const OstProto::VariableField &varField) const
+{
+
+    switch (varField.type()) {
+    case OstProto::VariableField::kCounter8:
+        varyCounter<quint8>(shortName(), buf, frameIndex, varField);
+        break;
+    case OstProto::VariableField::kCounter16:
+        varyCounter<quint16>(shortName(), buf, frameIndex, varField);
+        break;
+    case OstProto::VariableField::kCounter32:
+        varyCounter<quint32>(shortName(), buf, frameIndex, varField);
+        break;
+    default:
+        break;
+    }
+
+    return;
+}
+
