@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2010 Srivats P.
+Copyright (C) 2010, 2014 Srivats P.
 
 This file is part of "Ostinato"
 
@@ -21,32 +21,44 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "abstractprotocol.h"
 
 #include "protocol.pb.h"
+
 #include "mac.h"
-#include "payload.h"
-#include "eth2.h"    
+#include "vlan.h"
+#include "svlan.h"
+#include "vlanstack.h"
+
+// L2 Protos
 #include "dot3.h"    
 #include "llc.h"    
-#include "snap.h"    
 #include "dot2llc.h"
+#include "snap.h"    
 #include "dot2snap.h"
-#include "vlan.h"    
-#include "vlanstack.h"    
+#include "eth2.h"    
+
+// L3 Protos
 #include "arp.h"    
 #include "ip4.h"    
-#include "ip6.h"    
-#include "ip6over4.h"    
-#include "ip4over6.h"    
+#include "ip6.h"
 #include "ip4over4.h"    
+#include "ip4over6.h"    
+#include "ip6over4.h"    
 #include "ip6over6.h"    
+
+// L4 Protos
 #include "icmp.h"    
 #include "igmp.h"    
 #include "mld.h"    
 #include "tcp.h"    
 #include "udp.h"    
+
+// L5 Protos
 #include "textproto.h"
-#include "userscript.h"
+
+// Special Protos
 #include "hexdump.h"
+#include "payload.h"
 #include "sample.h"
+#include "userscript.h"
 
 ProtocolManager *OstProtocolManager;
 
@@ -58,41 +70,44 @@ ProtocolManager::ProtocolManager()
     registerProtocol(OstProto::Protocol::kMacFieldNumber,
             (void*) MacProtocol::createInstance);
 
+    registerProtocol(OstProto::Protocol::kVlanFieldNumber,
+            (void*) VlanProtocol::createInstance);
+    registerProtocol(OstProto::Protocol::kSvlanFieldNumber,
+            (void*) SVlanProtocol::createInstance);
+    registerProtocol(OstProto::Protocol::kVlanStackFieldNumber,
+            (void*) VlanStackProtocol::createInstance);
+
     registerProtocol(OstProto::Protocol::kEth2FieldNumber,
             (void*) Eth2Protocol::createInstance);
     registerProtocol(OstProto::Protocol::kDot3FieldNumber,
             (void*) Dot3Protocol::createInstance);
     registerProtocol(OstProto::Protocol::kLlcFieldNumber,
             (void*) LlcProtocol::createInstance);
-    registerProtocol(OstProto::Protocol::kSnapFieldNumber,
-            (void*) SnapProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kDot2LlcFieldNumber,
             (void*) Dot2LlcProtocol::createInstance);
+    registerProtocol(OstProto::Protocol::kSnapFieldNumber,
+            (void*) SnapProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kDot2SnapFieldNumber,
             (void*) Dot2SnapProtocol::createInstance);
 
-    registerProtocol(OstProto::Protocol::kSvlanFieldNumber,
-            (void*) SVlanProtocol::createInstance);
-    registerProtocol(OstProto::Protocol::kVlanFieldNumber,
-            (void*) VlanProtocol::createInstance);
-    registerProtocol(OstProto::Protocol::kVlanStackFieldNumber,
-            (void*) VlanStackProtocol::createInstance);
-
+    // Layer 3 Protocols
     registerProtocol(OstProto::Protocol::kArpFieldNumber,
             (void*) ArpProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kIp4FieldNumber,
             (void*) Ip4Protocol::createInstance);
     registerProtocol(OstProto::Protocol::kIp6FieldNumber,
             (void*) Ip6Protocol::createInstance);
-    registerProtocol(OstProto::Protocol::kIp6over4FieldNumber,
-            (void*) Ip6over4Protocol::createInstance);
-    registerProtocol(OstProto::Protocol::kIp4over6FieldNumber,
-            (void*) Ip4over6Protocol::createInstance);
+
     registerProtocol(OstProto::Protocol::kIp4over4FieldNumber,
             (void*) Ip4over4Protocol::createInstance);
+    registerProtocol(OstProto::Protocol::kIp4over6FieldNumber,
+            (void*) Ip4over6Protocol::createInstance);
+    registerProtocol(OstProto::Protocol::kIp6over4FieldNumber,
+            (void*) Ip6over4Protocol::createInstance);
     registerProtocol(OstProto::Protocol::kIp6over6FieldNumber,
             (void*) Ip6over6Protocol::createInstance);
 
+    // Layer 4 Protocols
     registerProtocol(OstProto::Protocol::kIcmpFieldNumber,
             (void*) IcmpProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kIgmpFieldNumber,
@@ -103,18 +118,20 @@ ProtocolManager::ProtocolManager()
             (void*) TcpProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kUdpFieldNumber,
             (void*) UdpProtocol::createInstance);
+
+    // Layer 5 Protocols
     registerProtocol(OstProto::Protocol::kTextProtocolFieldNumber,
             (void*) TextProtocol::createInstance);
 
+    // Special Protocols
     registerProtocol(OstProto::Protocol::kHexDumpFieldNumber,
             (void*) HexDumpProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kPayloadFieldNumber,
             (void*) PayloadProtocol::createInstance);
-
-    registerProtocol(OstProto::Protocol::kUserScriptFieldNumber,
-            (void*) UserScriptProtocol::createInstance);
     registerProtocol(OstProto::Protocol::kSampleFieldNumber,
             (void*) SampleProtocol::createInstance);
+    registerProtocol(OstProto::Protocol::kUserScriptFieldNumber,
+            (void*) UserScriptProtocol::createInstance);
 
     populateNeighbourProtocols();
 }
@@ -178,7 +195,10 @@ AbstractProtocol* ProtocolManager::createProtocol(int protoNumber,
     pc = (AbstractProtocol* (*)(StreamBase*, AbstractProtocol*))
         factory.value(protoNumber);
     
-    Q_ASSERT(pc != NULL);
+    Q_ASSERT_X(pc != NULL, 
+               __FUNCTION__, 
+               QString("No Protocol Creator registered for protocol %1")
+                    .arg(protoNumber).toAscii().constData());
 
     p = (*pc)(stream, parent);
 
