@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2010 Srivats P.
+Copyright (C) 2010, 2014 Srivats P.
 
 This file is part of "Ostinato"
 
@@ -20,47 +20,37 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #ifndef _RPC_SERVER_H
 #define _RPC_SERVER_H
 
-#include <google/protobuf/message.h>
-#include <google/protobuf/descriptor.h>
-#include <google/protobuf/service.h>
-#include <google/protobuf/io/zero_copy_stream_impl.h>
+#include "sharedprotobufmessage.h"
 
 #include <QTcpServer>
-#include <QTcpSocket>
 
-#include "pbrpccommon.h"
-#include "pbrpccontroller.h"
+// forward declaration
+namespace google {
+    namespace protobuf {
+        class Service;
+        class Message;
+    }
+}
 
-
-class RpcServer : public QObject
+class RpcServer : public QTcpServer
 {
     Q_OBJECT
 
-    QTcpServer *server;
-    QTcpSocket *clientSock;
-
-    ::google::protobuf::Service *service;
-    ::google::protobuf::io::CopyingInputStreamAdaptor  *inStream;
-    ::google::protobuf::io::CopyingOutputStreamAdaptor *outStream;
-
-    bool isPending;
-    int pendingMethodId;
-    QString errorString_;
-
 public:
     RpcServer();    //! \todo (LOW) use 'parent' param
     virtual ~RpcServer();
 
     bool registerService(::google::protobuf::Service *service,
         quint16 tcpPortNum);
-    QString errorString();
-    void done(PbRpcController *controller);
 
-private slots:
-     void when_newConnection();
-     void when_disconnected();
-     void when_dataAvail();
-    void when_error(QAbstractSocket::SocketError socketError);
+signals:
+    void notifyClients(int notifType, SharedProtobufMessage notifData);
+
+protected:
+    void incomingConnection(int socketDescriptor);
+
+private:
+    ::google::protobuf::Service *service;
 };
 
 #endif
