@@ -61,7 +61,7 @@ void AbstractPort::init()
 
 bool AbstractPort::modify(const OstProto::Port &port)
 {
-    bool ret = false;
+    bool ret = true;
 
     //! \todo Use reflection to find out which fields are set
     if (port.has_is_exclusive_control())
@@ -76,6 +76,10 @@ bool AbstractPort::modify(const OstProto::Port &port)
     if (port.has_transmit_mode())
         data_.set_transmit_mode(port.transmit_mode());
 
+    if (port.has_user_name()) {
+        data_.set_user_name(port.user_name());
+    }
+
     return ret;
 }    
 
@@ -172,7 +176,7 @@ void AbstractPort::updatePacketListSequential()
     {
         if (streamList_[i]->isEnabled())
         {
-            int len;
+            int len = 0;
             ulong n, x, y;
             ulong burstSize;
             double ibg = 0;
@@ -366,7 +370,7 @@ void AbstractPort::updatePacketListInterleaved()
         double numBursts = 0;
         double numPackets = 0;
 
-        quint64 _burstSize;
+        quint64 _burstSize = 0;
         double ibg = 0;
         quint64 _ibg1 = 0, _ibg2 = 0;
         quint64 _nb1 = 0, _nb2 = 0;
