@@ -20,40 +20,70 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "drone.h"
 
 #include "../common/protocolmanager.h"
+#include "settings.h"
 
 #include <google/protobuf/stubs/common.h>
 
+#include <QCoreApplication>
+#include <QFile>
+
 #ifdef Q_OS_UNIX
 #include <signal.h>
 #endif
 
 extern ProtocolManager *OstProtocolManager;
+extern char *version;
+extern char *revision;
 
+QSettings *appSettings;
 int myport;
 
 void cleanup(int /*signum*/)
 {
-    qApp->exit(-1);
+    QCoreApplication::instance()->exit(-1);
 }
 
 int main(int argc, char *argv[])
 {
     int exitCode = 0;
-    QApplication app(argc, argv);
-    Drone *drone = new Drone();
-    OstProtocolManager = new ProtocolManager();
-
-    app.setApplicationName(drone->objectName());
+    QCoreApplication app(argc, argv);
+    Drone *drone;
 
+    // TODO: command line options
+    // -v (--version)
+    // -h (--help)
+    // -p (--portnum)
     if (argc > 1)
         myport = atoi(argv[1]);
 
+    app.setApplicationName("Drone");
+    app.setOrganizationName("Ostinato");
+
+    /* (Portable Mode) If we have a .ini file in the same directory as the 
+       executable, we use that instead of the platform specific location
+       and format for the settings */
+    QString portableIni = QCoreApplication::applicationDirPath() 
+            + "/drone.ini";
+    if (QFile::exists(portableIni))
+        appSettings = new QSettings(portableIni, QSettings::IniFormat);
+    else
+        appSettings = new QSettings(QSettings::IniFormat, 
+                                    QSettings::UserScope,
+                                    app.organizationName(), 
+                                    app.applicationName().toLower());
+
+    drone = new Drone();
+    OstProtocolManager = new ProtocolManager();
+
     if (!drone->init())
     {
         exitCode = -1;
         goto _exit;
     }
 
+    qDebug("Version: %s", version);
+    qDebug("Revision: %s", revision);
+
 #ifdef Q_OS_UNIX
     struct sigaction sa;
     memset(&sa, 0, sizeof(sa));
@@ -64,10 +94,6 @@ int main(int argc, char *argv[])
         qDebug("Failed to install SIGINT handler. Cleanup may not happen!!!");
 #endif
 
-    drone->setWindowFlags(drone->windowFlags()
-        | Qt::WindowMaximizeButtonHint 
-        | Qt::WindowMinimizeButtonHint);
-    drone->showMinimized();
     exitCode = app.exec();
 
 _exit:
