@@ -98,7 +98,7 @@ PcapPort::PcapPort(int id, const char *device)
         if (strcmp(device, dev->name) == 0)
         {
 #ifdef Q_OS_WIN32
-            data_.set_name(QString("if%1 ").arg(id).toStdString());
+            data_.set_name(QString("if%1").arg(id).toStdString());
 #else
             if (dev->name)
                 data_.set_name(dev->name);
@@ -324,6 +324,7 @@ PcapPort::PortTransmitter::PortTransmitter(const char *device)
         Q_ASSERT_X(false, "PortTransmitter::PortTransmitter",
                 "This Win32 platform does not support performance counter");
 #endif
+    state_ = kNotStarted;
     returnToQIdx_ = -1;
     loopDelay_ = 0;
     stop_ = false;
@@ -490,7 +491,7 @@ void PcapPort::PortTransmitter::run()
 
     qDebug("packetSequenceList_.size = %d", packetSequenceList_.size());
     if (packetSequenceList_.size() <= 0)
-        return;
+        goto _exit;
 
     for(i = 0; i < packetSequenceList_.size(); i++) {
         qDebug("sendQ[%d]: rptCnt = %d, rptSz = %d, usecDelay = %ld", i, 
@@ -502,7 +503,9 @@ void PcapPort::PortTransmitter::run()
                 packetSequenceList_.at(i)->usecDuration_);
     }
 
-    for(i = 0; i < packetSequenceList_.size(); i++)
+    state_ = kRunning;
+    i = 0;
+    while (i < packetSequenceList_.size())
     {
 
 _restart:
@@ -562,10 +565,13 @@ _restart:
                     qDebug("error %d in sendQueueTransmit()", ret);
                     qDebug("overHead = %ld", overHead);
                     stop_ = false;
-                    return;
+                    goto _exit;
                 }
             }
         }
+
+        // Move to the next Packet Set
+        i += rptSz;
     }
 
     if (returnToQIdx_ >= 0)
@@ -583,12 +589,43 @@ _restart:
         i = returnToQIdx_;
         goto _restart;
     }
+
+_exit:
+    state_ = kFinished;
+}
+
+void PcapPort::PortTransmitter::start()
+{
+    // FIXME: return error
+    if (state_ == kRunning) {
+        qWarning("Transmit start requested but is already running!");
+        return;
+    }
+
+    state_ = kNotStarted;
+    QThread::start();
+
+    while (state_ == kNotStarted)
+        QThread::msleep(10);
 }
 
 void PcapPort::PortTransmitter::stop()
 {
-    if (isRunning())
+    if (state_ == kRunning) {
         stop_ = true;
+        while (state_ == kRunning)
+            QThread::msleep(10);
+    }
+    else {
+        // FIXME: return error
+        qWarning("Transmit stop requested but is not running!");
+        return;
+    }
+}
+
+bool PcapPort::PortTransmitter::isRunning()
+{
+    return (state_ == kRunning);
 }
 
 int PcapPort::PortTransmitter::sendQueueTransmit(pcap_t *p,
@@ -688,6 +725,7 @@ PcapPort::PortCapturer::PortCapturer(const char *device)
 {
     device_ = QString::fromAscii(device);
     stop_ = false;
+    state_ = kNotStarted;
 
     if (!capFile_.open())
         qWarning("Unable to open temp cap file");
@@ -713,7 +751,7 @@ void PcapPort::PortCapturer::run()
     if (!capFile_.isOpen())
     {
         qWarning("temp cap file is not open");
-        return;
+        goto _exit;
     }
 _retry:
     handle_ = pcap_open_live(device_.toAscii().constData(), 65535, 
@@ -732,13 +770,13 @@ _retry:
         {
             qDebug("%s: Error opening port %s: %s\n", __FUNCTION__,
                     device_.toAscii().constData(), errbuf);
-            return;
+            goto _exit;
         }
     }
 
     dumpHandle_ = pcap_dump_open(handle_, 
             capFile_.fileName().toAscii().constData());
-
+    state_ = kRunning;
     while (1)
     {
         int ret;
@@ -774,12 +812,43 @@ _retry:
     dumpHandle_ = NULL;
     handle_ = NULL;
     stop_ = false;
+
+_exit:
+    state_ = kFinished;
+}
+
+void PcapPort::PortCapturer::start()
+{
+    // FIXME: return error
+    if (state_ == kRunning) {
+        qWarning("Capture start requested but is already running!");
+        return;
+    }
+
+    state_ = kNotStarted;
+    QThread::start();
+
+    while (state_ == kNotStarted)
+        QThread::msleep(10);
 }
 
 void PcapPort::PortCapturer::stop()
 {
-    if (isRunning())
+    if (state_ == kRunning) {
         stop_ = true;
+        while (state_ == kRunning)
+            QThread::msleep(10);
+    }
+    else {
+        // FIXME: return error
+        qWarning("Capture stop requested but is not running!");
+        return;
+    }
+}
+
+bool PcapPort::PortCapturer::isRunning()
+{
+    return (state_ == kRunning);
 }
 
 QFile* PcapPort::PortCapturer::captureFile()
